@@ -7,6 +7,7 @@
  * Support : support@synerty.com
 """
 import json
+from typing import Any, Dict, Optional
 
 from vortex.Tuple import Tuple, addTupleType, TupleField
 
@@ -14,11 +15,13 @@ from vortex.Tuple import Tuple, addTupleType, TupleField
 @addTupleType
 class TupleSelector(Tuple):
     __tupleType__ = "vortex.TupleSelector"
+    __slots__ = ["name", "selector"]
 
-    name = TupleField(comment="The tuple name this selector is for")
-    selector = TupleField(comment="The values to select")
+    # name: str = TupleField(comment="The tuple name this selector is for")
+    # selector: Dict[str, Any] = TupleField(comment="The values to select")
 
-    def __init__(self, name: str = None, selector: dict = None):
+    def __init__(self, name: Optional[str] = None, selector: Optional[Dict] = None) -> None:
+        Tuple.__init__(self)
         self.name = name
         self.selector = selector if selector else {}
 
@@ -37,3 +40,18 @@ class TupleSelector(Tuple):
         """
         return json.dumps({'name': self.name,
                            'selector': self.selector}, sort_keys=True)
+
+    @classmethod
+    def fromJsonStr(self, jsonStr:str) -> "TupleSelector":
+        """ From Json Str
+
+        This method creates a new c{TupleSelector} from the ordered json string dumped
+        from .toJsonStr
+
+        """
+        data = json.loads(jsonStr)
+        return TupleSelector(name=data["name"], selector=data["selector"])
+
+    def __repr__(self):
+        return self.toJsonStr()
+
