@@ -1,5 +1,6 @@
 import logging
 from copy import copy
+from typing import Optional
 
 from twisted.internet.defer import TimeoutError, inlineCallbacks, Deferred
 from twisted.python.failure import Failure
@@ -10,6 +11,8 @@ from vortex.PayloadEnvelope import PayloadEnvelope
 from vortex.PayloadResponse import PayloadResponse
 from vortex.VortexABC import SendVortexMsgResponseCallable
 from vortex.VortexFactory import VortexFactory
+from vortex.handler.TupleActionProcessor import TupleActionProcessor, \
+    TupleActionProcessorDelegateABC
 
 logger = logging.getLogger(__name__)
 
@@ -24,7 +27,8 @@ class TupleActionProcessorProxy:
 
     def __init__(self, tupleActionProcessorName,
                  proxyToVortexName: str,
-                 additionalFilt=None) -> None:
+                 additionalFilt=None,
+                 acceptOnlyFromVortex: Optional[str] = None) -> None:
         """ Constructor
 
         :param tupleActionProcessorName: The name of this and the other action handler
@@ -32,25 +36,64 @@ class TupleActionProcessorProxy:
         :param additionalFilt: Any additional filter keys that are required
         """
         self._proxyToVortexName = proxyToVortexName
+        self._delegateProcessor = TupleActionProcessor(
+            tupleActionProcessorName=tupleActionProcessorName,
+            usedForProxy__=True
+        )
 
         self._filt = dict(name=tupleActionProcessorName, key="tupleActionProcessorName")
 
         if additionalFilt:
             self._filt.update(additionalFilt)
 
-        self._endpoint = PayloadEndpoint(self._filt, self._process)
+        self._endpoint = PayloadEndpoint(self._filt, self._process,
+                                         acceptOnlyFromVortex=acceptOnlyFromVortex)
 
     def shutdown(self):
         self._endpoint.shutdown()
 
+    def setDelegate(self, tupleName: str, processor: TupleActionProcessorDelegateABC):
+        """ Add Tuple Action Processor Delegate
+
+        :param tupleName: The tuple name to process actions for.
+        :param processor: The processor to use for processing this tuple name.
+
+        """
+        self._delegateProcessor.setDelegate(tupleName, processor)
+
     @inlineCallbacks
     def _process(self, payloadEnvelope: PayloadEnvelope, vortexName: str,
-                 sendResponse: SendVortexMsgResponseCallable, **kwargs):
+                 sendResponse: SendVortexMsgResponseCallable, **kwargs) -> None:
 
         # Ignore responses from the backend, these are handled by PayloadResponse
         if vortexName == self._proxyToVortexName:
             return
 
+        # Shortcut the logic, so that we don't decode the payload unless we need to.
+        if not self._delegateProcessor.delegateCount:
+            yield self._processForProxy(payloadEnvelope, vortexName, sendResponse)
+            return
+
+        # If we have local processors, then work out if this tupleAction is meant for
+        # the local processor.
+        payload = yield payloadEnvelope.decodePayloadDefer()
+
+        assert len(payload.tuples) == 1, (
+                "TupleActionProcessor:%s Expected 1 tuples, received %s" % (
+            self._tupleActionProcessorName, len(payload.tuples)))
+
+        tupleAction = payload.tuples[0]
+
+        if self._delegateProcessor.hasDelegate(tupleAction.tupleName()):
+            self._delegateProcessor._processTupleAction(payloadEnvelope.filt, sendResponse, tupleAction)
+            return
+
+        # Else, Just send it on to the delegate we're proxying for (the backend)
+        yield self._processForProxy(payloadEnvelope, vortexName, sendResponse)
+
+    @inlineCallbacks
+    def _processForProxy(self, payloadEnvelope: PayloadEnvelope, vortexName: str,
+                         sendResponse: SendVortexMsgResponseCallable, **kwargs):
         # Keep a copy of the incoming filt, in case they are using PayloadResponse
         responseFilt = copy(payloadEnvelope.filt)
 
